/*
  The MIT License (MIT)

  Copyright (c) 2014-2017 Marc de Verdelhan & respective authors (see AUTHORS)

  Permission is hereby granted, free of charge, to any person obtaining a copy of
  this software and associated documentation files (the "Software"), to deal in
  the Software without restriction, including without limitation the rights to
  use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
  the Software, and to permit persons to whom the Software is furnished to do so,
  subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
  FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
  IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package org.ta4j.core.indicators;

import org.ta4j.core.Decimal;
import org.ta4j.core.Indicator;

/**
 * Triple exponential moving average indicator.
 * </p>
 * a.k.a TRIX
 * </p>
 * TEMA needs "3 * period - 2" of data to start producing values in contrast to
 * the period samples needed by a regular EMA.
 * </p>
 * see https://en.wikipedia.org/wiki/Triple_exponential_moving_average
 */
public class TripleEMAIndicator extends CachedIndicator<Decimal> {

    private static final long serialVersionUID = -3091675249185831978L;

    private final int timeFrame;
    private final EMAIndicator ema;
    private final EMAIndicator emaEma;
    private final EMAIndicator emaEmaEma;

    /**
     * Constructor.
     * 
     * @param indicator the indicator
     * @param timeFrame the time frame
     */
    public TripleEMAIndicator(Indicator<Decimal> indicator, int timeFrame) {
        super(indicator);
        this.timeFrame = timeFrame;
        this.ema = new EMAIndicator(indicator, timeFrame);
        this.emaEma = new EMAIndicator(ema, timeFrame);
        this.emaEmaEma = new EMAIndicator(emaEma, timeFrame);
    }

    @Override
    protected Decimal calculate(int index) {
    	
    		// trix = 3 * ema - 3 * emaEma + emaEmaEma 
        return Decimal.THREE.multipliedBy(
        		ema.getValue(index)
        			.minus(emaEma.getValue(index)))
        			.plus(emaEmaEma.getValue(index));
    }
    
    @Override
    public String toString() {
        return getClass().getSimpleName() + " timeFrame: " + timeFrame;
    }
}
